@@ -13,35 +13,57 @@ has default => sub {
   require Mojolicious::Plugin::RoutesAuthDBI::Schema;
   {
   auth => {
-    stash_key => $pkg,
-    current_user_fn => 'auth_user',
+    stash_key => $pkg."__user__",
+    current_user_fn => 'auth_user',# helper
     load_user => \&load_user,
     validate_user => \&validate_user,
   },
+  
   access => {
     namespace => $pkg,
     module => 'Access',
     fail_auth_cb => sub {
-      shift->render(format=>'txt', text=>"Deny access at auth step. Please sign in.\n");
+      shift->render(status => 401, format=>'txt', text=>"Please sign in.\n");
     },
     fail_access_cb => sub {
-      shift->render(format=>'txt', text=>"You don`t have access on this route (url, action).\n");
+      shift->render(status => 403, format=>'txt', text=>"You don`t have access on this route (url, action).\n");
     },
     import => [qw(load_user validate_user)],
   },
+  
   admin => {
     namespace => $pkg,
     controller => 'Admin',
     prefix => lc($self->conf->{admin}{controller} || 'admin'),
     trust => hmac_sha1_sum('admin', $self->app->secrets->[0]),
+    role_admin => 'administrators',
   },
+  
   oauth => {
     namespace => $pkg,
-    controller => 'OAuth2',
+    controller => 'OAuth',
     fail_auth_cb => sub {shift->render(format=>'txt', text=>"@_")},
   },
+  
   template => $Mojolicious::Plugin::RoutesAuthDBI::Schema::defaults,
-}};
+  
+  model_namespace => $pkg.'::Model',
+  
+  guest => {# from Mojolicious::Plugin::Authentication
+    #~ autoload_user => 1,
+    session_key => 'guest_data',
+    stash_key => $pkg."__guest__",
+    #~ current_user_fn => 'current_guest',# helper
+    #~ load_user => \&load_guest,
+    #~ validate_user => not need
+    #~ fail_render => not need
+    #######end Mojolicious::Plugin::Authentication conf#######
+    namespace => $pkg,
+    module => 'Guest',
+    #~ import => [qw(load_guest)],
+  },
+  };
+};
 
 has merge_conf => sub {#hashref
   my $self = shift;
@@ -67,14 +89,23 @@ has admin => sub {# object
 has oauth => sub {
   my $self = shift;
   my $conf = $self->merge_conf->{'oauth'};
-  load_class($conf)->init(%$conf, app=>$self->app, plugin=>$self,);
+  load_class($conf)->init(%$conf, app=>$self->app, plugin=>$self, model=>$self->model($conf->{controller}),);
 };
 
-has model => sub {
-  my $m = { map {$_ => load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::$_")->new} qw(Profiles Namespaces Routes Refs Controllers Actions Roles Logins) };
+has guest => sub {# object
+  my $self = shift;
+  my $conf = $self->merge_conf->{'guest'};
+  my $class = load_class($conf);
+  #~ $class->import( @{ $conf->{import} });
   
+  $class->new( %$conf, app=>$self->app, plugin=>$self, model=>$self->model($conf->{module}), );# plugin=>$self,
 };
 
+#~ has model => sub {
+  #~ my $m = { map {$_ => load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::$_")->new} qw(Profiles Namespaces Routes Refs Controllers Actions Roles Logins) };
+  
+#~ };
+
 sub register {
   my $self = shift;
   $self->app(shift);
@@ -109,6 +140,8 @@ sub register {
     $access->apply_route($_) for $admin->self_routes;
   }
   
+  $self->guest
+    if $self->conf->{guest};
   
   $self->app->helper('access', sub {$access});
   
@@ -126,27 +159,8 @@ sub cond_access {# add_condition
   
   my $auth_helper = $conf->{auth}{current_user_fn};
   my $u = $c->$auth_helper;
-  $app->log->debug(sprintf(qq[Access allow [%s] for {auth}=false],
-    $route->pattern->unparsed,
-  ))
-    and return 1 # не проверяем доступ
-    unless $args->{auth};
-  
   my $fail_auth_cb = $conf->{access}{fail_auth_cb};
   
-  # не авторизовался
-  $self->deny_log($route, $args, $u)
-    and $c->$fail_auth_cb()
-    and return undef
-    unless $u;
-  
-  # допустить если {auth=>'only'}
-  $app->log->debug(sprintf(qq[Access allow [%s] for {auth}='only'],
-    $route->pattern->unparsed,
-  ))
-    and return 1
-    if lc($args->{auth}) eq 'only';
-  
   if (ref $args eq 'CODE') {
     $args->($u, @_)
       or $self->deny_log($route, $args, $u)
@@ -157,6 +171,34 @@ sub cond_access {# add_condition
     ));
     return 0x01;
   }
+  
+  $app->log->debug(
+    sprintf(qq[Access allow [%s] for none {auth} and none {role} and none {guest}], $route->pattern->unparsed)
+  )
+    and return 1 # не проверяем доступ
+    unless $args->{auth} || $args->{role} || $args->{guest};
+  
+  
+  if ($args->{guest}) {#  && $args->{auth} =~ m'\bguest\b'i
+    $app->log->debug(sprintf(qq[Access allow [%s] for {guest}],
+        $route->pattern->unparsed,
+      ))
+      and return 1
+      if $self->guest->is_guest($c);
+  }
+  
+  # не авторизовался
+  $self->deny_log($route, $args, $u)
+    and $c->$fail_auth_cb()
+    and return undef
+    unless $u && $u->{id};
+  
+  # допустить если {auth=>'only'}
+  $app->log->debug(sprintf(qq[Access allow [%s] for {auth}=~'only'],
+    $route->pattern->unparsed,
+  ))
+    and return 1
+    if $args->{auth} && $args->{auth} =~ m'\bonly\b'i;
 
   my $id2 = [$u->{id}, map($_->{id}, grep !$_->{disable},@{$u->roles})];
   my $id1 = [grep $_, @$args{qw(id route_id action_id controller_id namespace_id)}];
@@ -273,7 +315,19 @@ sub deny_log {
   );
 }
 
-our $VERSION = '0.785';
+sub model {
+  my ($self, $name) = @_;
+  my $ns = $self->merge_conf->{'model_namespace'};
+  my $class =  load_class(namespace => $ns, module=> $name)
+    or die "Model module [$name] not found at namespace [$ns] or has errors";
+  
+  $class->new; # синглетоны в общем
+  
+  #~ my $m = { map {$_ => load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::$_")->new} qw(Profiles Namespaces Routes Refs Controllers Actions Roles Logins) };
+  
+};
+
+our $VERSION = '0.830';
 
 =pod
 
@@ -281,17 +335,19 @@ our $VERSION = '0.785';
 
 Доброго всем
 
+¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
+
 =head1 Mojolicious::Plugin::RoutesAuthDBI
 
-¡ ¡ ¡ ALL GLORY TO GLORIA ! ! !
+Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> and OAuth2 by L<Mojolicious::Plugin::OAuth2>.
 
 =head1 VERSION
 
-0.785
+0.830
 
 =head1 NAME
 
-Mojolicious::Plugin::RoutesAuthDBI - from DBI tables does generate routes, make authentication and make restrict access (authorization) to requests. Plugin makes an auth operations throught the plugin L<Mojolicious::Plugin::Authentication> and OAuth2 by L<Mojolicious::Plugin::OAuth2>.
+Mojolicious::Plugin::RoutesAuthDBI - from DBI tables does generate routes, make authentication and make restrict access (authorization) to app routes.
 
 =head1 DB DESIGN DIAGRAM
 
@@ -305,7 +361,9 @@ First of all you will see L<SVG|https://github.com/mche/Mojolicious-Plugin-Route
     access => {...},
     admin => {...},
     oauth => {...},
+    guest => {...},
     template => {...},
+    model_namespace=>...,
   );
 
 
@@ -327,6 +385,7 @@ Hashref options pass to base plugin L<Mojolicious::Plugin::Authentication>.
 By default the option:
 
   current_user_fn => 'auth_user',
+  stash_key => "Mojolicious::Plugin::RoutesAuthDBI__user__",
     
 The options:
 
@@ -382,7 +441,7 @@ See L<Mojolicious::Plugin::RoutesAuthDBI::Admin> for detail options list.
 Hashref options for oauth controller. By default the builtin module:
 
   oauth => {
-    controller => 'OAuth2',
+    controller => 'OAuth',
     namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
     ...,
   },
@@ -395,7 +454,29 @@ You might define your own controller by passing options:
     ...,
   },
 
-See L<Mojolicious::Plugin::RoutesAuthDBI::OAuth2> for detail options list.
+See L<Mojolicious::Plugin::RoutesAuthDBI::OAuth> for detail options list.
+
+=head3 guest
+
+Hashref options for guest module. Defaults are:
+
+  guest => {
+    namespace => 'Mojolicious::Plugin::RoutesAuthDBI',
+    module => 'Guest',
+    session_key => 'guest_data',
+    stash_key => "Mojolicious::Plugin::RoutesAuthDBI__guest__",
+    
+  },
+
+Disable guest module usage:
+
+  guest => undef, # or none in config
+
+See L<Mojolicious::Plugin::RoutesAuthDBI::Guest>
+
+=head3 model_namespace
+
+Where are your models place. Default to "Mojolicious::Plugin::RoutesAuthDBI::Model".
 
 =head3 template
 
@@ -423,6 +504,16 @@ Heart of this plugin! This condition apply for all db routes even if column auth
   # same as
   # $r->route('/foo')->...->over(authenticated => 1)->...; # see Mojolicious::Plugin::Authentication
 
+=item * Allow for guest
+
+  $r->route('/foo')->...->over(access=>{guest=>1})->...;
+  
+To makes guest session:
+
+  $c->access->plugin->guest->store($c, {<...some data...>});
+
+See L<Mojolicious::Plugin::RoutesAuthDBI::Guest>
+
 =item * Route accessible if profile roles assigned to either B<loadable> namespace or controller 'Bar.pm' (which assigned neither namespece on db or assigned to that loadable namespace) or action 'bar' on controller Bar.pm (action record in db table actions):
 
   $r->route('/bar-bar-any-namespace')->to('bar#bar',)->over(access=>{auth=>1})->...;
