@@ -3,9 +3,11 @@ use Mojo::Base -base;
 use Exporter 'import'; 
 our @EXPORT_OK = qw(load_user validate_user);
 use Mojolicious::Plugin::RoutesAuthDBI::Util qw(load_class);
+use Mojo::Util qw(md5_sum);
 
 has [qw(app plugin)];
 
+#~ has model_profiles => sub { shift->plugin->model('Profiles') };
 #~ has model => sub {
   #~ { map $_ => load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::$_")->new, qw(Profiles Namespaces Routes Refs Controllers Actions Roles) }
 #~ };
@@ -18,9 +20,15 @@ sub load_user {# import for Mojolicious::Plugin::Authentication
   my ($c, $uid) = @_;
   
   my $p = load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::Profiles")->get_profile($uid, undef);
-  $c->app->log->debug("Loading profile by id=$uid ". ($p->{id} ? 'success' : 'failed'));
-  $p->{pass} = '**********************';
-  return $p;
+  #~ my $p = $c->model_profiles->get_profile($uid, undef);
+  if ($p->{id}) {
+    $c->app->log->debug("Loading profile by id=$uid success");
+    $p->{pass} = '**********************';
+    return $p;
+  }
+  $c->app->log->debug("Loading profile by id=$uid failed");
+  
+  return undef;
 }
 
 sub validate_user {# import for Mojolicious::Plugin::Authentication
@@ -28,18 +36,27 @@ sub validate_user {# import for Mojolicious::Plugin::Authentication
   
   return $extradata->{id}
     if $extradata && $extradata->{id};
-    
+  
   if (my $p = load_class("Mojolicious::Plugin::RoutesAuthDBI::Model::Profiles")->get_profile(undef, $login)) {
+  #~ if (my $p = $c->model_profiles->get_profile(undef, $login)) {
     $c->app->log->debug("Success authenticate by login[$login]/pass[$pass] for profile id[$p->{id}]")
       and return $p->{id}
-      if $p->{pass} eq $pass  && !$p->{disable};
+      if ($p->{pass} eq $pass || $p->{pass} eq md5_sum($pass))
+        && !$p->{disable};
+    
+    $c->app->log->debug("Failure authenticate by login[$login]/pass[$pass]:[@{[md5_sum($pass)]}] for profile id[$p->{id}:$p->{pass}]");
+    
+    return undef;
   }
+  
+  $c->app->log->debug("Failure authenticate by login[$login]/pass[$pass]:[@{[md5_sum($pass)]}]");
+  
   return undef;
 }
 
 sub apply_ns {# Plugin
   my ($self,) = @_;
-  my $ns = $self->plugin->model->{Namespaces}->app_ns;
+  my $ns = $self->plugin->model('Namespaces')->app_ns;
   return unless @$ns;
   my $r = $self->app->routes;
   push @{ $r->namespaces() }, $_->{namespace} for @$ns;
@@ -74,13 +91,23 @@ sub apply_route {# meth in Plugin
   # STEP ACCESS
   $nr->over(access => $r_hash);
   
-  if ( $r_hash->{action} ) {
+# Controller and action in Mojolicious::Routes::Route->to
+    #~ elsif ($shortcut =~ /^([\w\-:]+)?\#(\w+)?$/) {
+      #~ $defaults{controller} = $1 if defined $1;
+      #~ $defaults{action}     = $2 if defined $2;
+    #~ }
+    
+  my %ns = (namespace => $r_hash->{namespace})
+    if $r_hash->{namespace};
+  my %controll = (controller=>$r_hash->{controller},)
+    if $r_hash->{controller};
 
-    my %ns = (namespace => $r_hash->{namespace})
-      if $r_hash->{namespace};
-      
-    if ( $r_hash->{action} =~ /#/ ) { $nr->to($r_hash->{action}, %ns); }
-    else { $nr->to(controller=>$r_hash->{controller}, action => $r_hash->{action}, %ns,); }
+  if ($r_hash->{to}) {
+    $nr->to($r_hash->{to}, %controll, %ns); 
+  } elsif ( $r_hash->{action} ) {
+    
+    if ( $r_hash->{action} =~ /#/ ) { $nr->to($r_hash->{action}, %controll, %ns); }
+    else { $nr->to( action => $r_hash->{action}, %controll, %ns,); }
     
   } elsif ( $r_hash->{callback} ) {
     
@@ -92,44 +119,117 @@ sub apply_route {# meth in Plugin
   } else {
     die "No defaults for route: ", $self->app->dumper($r_hash);
   }
-  $nr->name($r_hash->{name}) if $r_hash->{name};
+  $nr->name($r_hash->{name})
+    if $r_hash->{name};
   #~ $self->app->log->debug("$pkg generate the route from data row [@{[$self->app->dumper($r_hash) =~ s/\n/ /gr]}]");
   return $nr;
 }
 
 sub routes {
   my ($self,) = @_;
-  $self->plugin->model->{Routes}->routes;
+  $self->plugin->model('Routes')->routes;
 }
 
 sub access_explicit {# i.e. by refs table
   my ($self, $id1, $id2,) = @_;
-  return scalar $self->plugin->model->{Refs}->cnt($id1, $id2);
+  return scalar $self->plugin->model('Refs')->cnt($id1, $id2);
 }
 
 
 sub access_namespace {#implicit
   my ($self, $namespace, $id2,) = @_;
-  return scalar $self->plugin->model->{Namespaces}->access($namespace, $id2);
+  return scalar $self->plugin->model('Namespaces')->access($namespace, $id2);
 }
 
 sub access_controller {#implicit
   my ($self, $namespace, $controller, $id2,) = @_;
-  my $c = $self->plugin->model->{Controllers}->controller_ns( $controller, ($namespace) x 2,)
+  my $c = $self->plugin->model('Controllers')->controller_ns( $controller, ($namespace) x 2,)
     or return undef;
   $self->access_explicit([$c->{id}], $id2);
 }
 
 sub access_action {#implicit
   my ($self, $namespace, $controller, $action, $id2,) = @_;
-  my $c = $self->plugin->model->{Controllers}->controller_ns( $controller, ($namespace) x 2,)
+  my $c = $self->plugin->model('Controllers')->controller_ns( $controller, ($namespace) x 2,)
     or return undef;
-  return scalar $self->plugin->model->{Actions}->access( $c->{id}, $action, $id2);
+  return scalar $self->plugin->model('Actions')->access( $c->{id}, $action, $id2);
 }
 
 sub access_role {#implicit
   my ($self, $role, $id2,) = @_;
-  return scalar $self->plugin->model->{Roles}->access($role =~ /\D/ ? (undef, $role) : ($role, undef), $id2);
+  return scalar $self->plugin->model('Roles')->access($role =~ /\D/ ? (undef, $role) : ($role, undef), $id2);
+}
+
+my $Mojo_Util_loaded;
+sub auth_cookie {
+  my ($self, $c, $value, $name) = @_;
+  $name ||= $c->app->sessions->cookie_name;
+  return $c->cookie($name)#'mojolicious'
+    unless $value;
+  
+  if ($value =~ s/--([^\-]+)$//) {
+    my $signature = $1;
+    load_class 'Mojo::Util'
+      unless $Mojo_Util_loaded++;
+
+    my $valid;
+    my $secrets = $c->app->secrets;
+    for my $secret (@$secrets) {
+      my $check = Mojo::Util::hmac_sha1_sum($value, $secret);
+      ++$valid
+        and last
+        if Mojo::Util::secure_compare($signature, $check);
+    }
+    
+    $c->app->log->warn(qq{Cookie [$value] has a bad signature})
+      and return undef
+      unless $valid;
+    
+  } else {
+    $c->app->log->warn(qq{Cookie [$value] is not signed})
+      and return undef;
+  }
+
+  my $session = $c->app->sessions->deserialize->(Mojo::Util::b64_decode $value)
+    or $c->app->log->warn(qq{Cookie [$value] couldnt deserialize})
+    and return undef;
+  
+  my $key =  $self->plugin->merge_conf->{auth}{session_key} || 'auth_data';
+  my $profile_id = $session->{$key}
+    or $c->app->log->warn(qq{Cookie [$value] doesnt has profile id})
+    and return undef;
+  #~ $c->app->log->fatal($c->dumper($c->stash->{'mojo.session'}), $c->stash->{'mojo.active_session'});
+  #~ warn $c->auth_user || 'none auth';
+  $c->authenticate(undef, undef, {id=> $profile_id}); # session only store
+  #~ $c->app->sessions->store($c);
+  #~ $c->app->log->fatal($c->dumper($c->stash->{'mojo.session'}), $c->stash->{'mojo.active_session'});
+  #~ $c->app->log->fatal($_->name, $_->{value}) for @{$c->res->cookies};
+  #~ $c->app->log->fatal( $c->dumper($c->res->cookies) );
+  #~ $c->app->log->fatal( $c->dumper($c->res->headers) );
+  #~ $c->res->headers->set_cookie('foo=bar; path=/');
+  
+  my $old_set_cookie = $c->res->headers->set_cookie;# && scalar @{$c->res->headers->{'set-cookie'}};
+  #~ $c->app->log->fatal( $c->dumper($old_set_cookie) );
+  #~ $c->app->log->fatal($c->cookie('mojolicious'));
+  $c->app->sessions->store($c);
+  #~ $c->app->log->fatal( $c->dumper( $c->res->headers->to_hash) );
+  #~ my $cc = scalar @{$c->res->cookies};
+  #~ $c->app->log->fatal($c->cookie('mojolicious'));
+  #~ $c->app->log->fatal($_->name, $_->value) for @{$c->res->cookies});
+  #~ my $new_cookie = (grep($_->name eq $name, @{$c->res->cookies}))[0];
+  my $new_cookie = pop @{$c->res->cookies};
+  #~ $c->app->log->fatal( $c->dumper($c->res->cookies) );
+  $c->res->headers->set_cookie($old_set_cookie // 'auth=ok; path=/');
+  #~ $set_cookie ? pop @{$c->res->headers->{'set-cookie'}} : delete $c->res->headers->{'set-cookie'};
+  #~ $c->app->log->fatal( $c->dumper($c->res->headers) );
+  #~ $c->app->log->fatal($c->dumper($c->stash->{'mojo.session'}), $c->stash->{'mojo.active_session'});
+  
+  my $profile = $c->auth_user;
+  $profile->auth_cookie($new_cookie->value)
+    #~ and unshift(@a,$x,$y)   splice(@{$c->res->cookies},0,0,$x,$y)
+    if $new_cookie;
+  
+  return $profile;
 }
 
 1;
@@ -198,82 +298,91 @@ See detail L<Mojolicious::Plugin::RoutesAuthDBI#access>
 
 =head1 OPTIONS for plugin
 
-=over 4
+=head2 namespace
 
-=item * B<namespace> - default 'Mojolicious::Plugin::RoutesAuthDBI',
+Default 'Mojolicious::Plugin::RoutesAuthDBI'.
 
-=item * B<module> - default 'Access' (this module),
+=head2 module
+
+Default 'Access' (this module).
 
 Both above options determining the module which will play as manager of authentication, accessing and generate routing from DBI source.
 
-=item * B<fail_auth_cb> = sub {my $c = shift;...}
+=head2 fail_auth_cb
+
+  fail_auth_cb => sub {my $c = shift;...}
 
 This callback invoke when request need auth route but authentication was failure.
 
-=item * B<fail_access_cb> = sub {my ($c, $route, $r_hash, $u) = @_;...}
+=head2 fail_access_cb
+
+  fail_access_cb => sub {my ($c, $route, $r_hash, $u) = @_;...}
 
 This callback invoke when request need auth route but access was failure. $route - L<Mojolicious::Routes::Route> object, $r_hash - route hashref db item, $u - useer hashref.
 
-=back
 
 =head1 EXPORT SUBS
 
-=over 4
+=head2 load_user($c, $uid)
 
-=item * B<load_user($c, $uid)> - fetch user record from table profiles by COOKIES. Import for Mojolicious::Plugin::Authentication. Required.
+Fetch user record from table profiles by COOKIES. Import for Mojolicious::Plugin::Authentication. Required.
 
-=item * B<validate_user($c, $login, $pass, $extradata)> - fetch login record from table logins by Mojolicious::Plugin::Authentication. Required. If hashref $extradata->{id} then no fetch and $extradata->{id} will return.
+=head2 validate_user($c, $login, $pass, $extradata)
 
-=back
+Fetch login record from table logins by Mojolicious::Plugin::Authentication. Required. If hashref $extradata->{id} then no fetch and $extradata->{id} will return.
 
-=head1 METHODS NEEDS IN PLUGIN
+=head1 METHODS
 
-=over 4
+As child of L<Mojolicious::Controller> inherits all parent methods and following ones:
 
-=item * B<new(app=> ..., plugin => ...)>
+=head2 new(app=> ..., plugin => ...)
 
 Return new access object.
 
-=item * B<apply_ns()>
+=head2 apply_ns()
 
 Select from db table I<namespaces> ns thus app_ns=1 and push them to $app->namespaces()
 
-=item * B<apply_route($r_hash)>
+=head2 apply_route($r_hash)
 
 Heart of routes generation from db tables and not only. Insert to app->routes an hash item $r_hash. DB schema specific. Return new Mojolicious route.
 
-=item * B<routes()>
+=head2 routes()
 
 Fetch records for apply_routes. Must return arrayref of hashrefs routes.
 
-=item * B<access_explicit($id1, $id2)>
+=head2 access_explicit($id1, $id2)
 
 Check access to route ($id1 arrayref - either route id or action id or controller id or namespace id) by roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
 
-=item * B<access_namespace($namespace, $id2)>
+=head2 access_namespace($namespace, $id2)
 
 Check implicit access to route by $namespace for profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this namespace.
 
-=item * B<access_controller($namespace, $controller, $id2)>
+=head2 access_controller($namespace, $controller, $id2)
 
 Check implicit access to route by $namespace and $controller for profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access to all actions of this controller.
 
-=item * B<access_action($namespace, $controller, $action, $id2)>
+=head2 access_action($namespace, $controller, $action, $id2)
 
 Check implicit access to route by $namespace and $controller and $action for profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access to this action.
 
-=item * B<access_role($role, $id2)>
+=head2 access_role($role, $id2)
 
 Check implicit access to route by $role (id|name) and profile roles ids ($id2 arrayref). Must return false for deny access or true - allow access.
 
-=back
+=head2 auth_cookie($c, $cookie_value, $cookie_name)
+
+Returns C<< $c->cookie($cookie_name) >> unless $cookie_value.
+
+Returns authenticate profile for $cookie_value. I use this method for cordova mobile app then cookie lost on any reasons.
+
+C<< $cookie_name >> has defaults to C<< $c->app->sessions->cookie_name >>
 
 =head1 SEE ALSO
 
 L<Mojolicious::Plugin::RoutesAuthDBI>
 
-L<Mojolicious::Plugin::RoutesAuthDBI::Sth>
-
 =head1 AUTHOR
 
 Михаил Че (Mikhail Che), C<< <mche [on] cpan.org> >>
