@@ -1,9 +1,10 @@
 package Mojolicious::Plugin::RoutesAuthDBI::Admin;
 use Mojo::Base 'Mojolicious::Controller';
+#~ use Mojo::Util qw(md5_sum);
 
 my $pkg = __PACKAGE__;
 my ($Init);
-has [qw(plugin controller namespace prefix trust)];
+has [qw(plugin controller namespace prefix trust role_admin)];
 
 sub init {# from plugin! init Class vars
   state $self = shift->SUPER::new(@_);
@@ -43,7 +44,7 @@ sub sign {
     and return;
     
   
-  $c->render(format=>'txt', text=>__PACKAGE__ . "\n\nBad sign!!! Try again");
+  $c->render(format=>'txt', text=>__PACKAGE__ . "\n\nBad sign! Try again");
 }
 
 sub signout {
@@ -55,10 +56,16 @@ sub signout {
   
 }
 
+sub out {# выход с редиректом
+  my $c = shift;
+  $c->logout;
+  $c->redirect_to($c->param('redirect') || '/');
+}
+
 sub users {
   my $c = shift;
   
-  my $p = $Init->plugin->model->{Profiles}->profiles;
+  my $p = $Init->plugin->model('Profiles')->profiles;
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -75,7 +82,7 @@ sub new_user {
   my ($login, $pass) = $c->vars(qw'login pass');
   
   my $r;
-  ($r = $Init->plugin->model->{Profiles}->get_profile(undef, $login))
+  ($r = $Init->plugin->model('Profiles')->get_profile(undef, $login))
     and $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -87,11 +94,11 @@ TXT
     and ($r->{not_new} = '!')
     and return $r;
   
-  $r = $Init->plugin->model->{Logins}->new_login($login, $pass);
+  $r = $Init->plugin->model('Logins')->new_login($login, $pass);
   
-  my $p = $Init->plugin->model->{Profiles}->new_profile([$login],);
+  my $p = $Init->plugin->model('Profiles')->new_profile([$login],);
   
-  $Init->plugin->model->{Refs}->refer($p->{id} => $r->{id});
+  $Init->plugin->model('Refs')->refer($p->{id} => $r->{id});
   
   @$p{qw(login pass)} = @$r{qw(login pass)};
   
@@ -112,25 +119,25 @@ sub trust_new_user {
   my $u = $c->new_user;
   
   # ROLE
-  my $rl = $Init->plugin->model->{Roles}->get_role(undef, 'admin')
-    || $Init->plugin->model->{Roles}->new_role('admin');
+  my $rl = $Init->plugin->model('Roles')->get_role(undef, $Init->role_admin || $Init->controller)
+    || $Init->plugin->model('Roles')->new_role($Init->role_admin || $Init->controller);
   
   # REF role->user
-  my $ru = $Init->plugin->model->{Refs}->refer($rl->{id} => $u->{id});
+  my $ru = $Init->plugin->model('Refs')->refer($rl->{id} => $u->{id});
   
   # CONTROLLER
-  my $cc = $Init->plugin->model->{Controllers}->controller_ns($Init->{controller}, ($Init->{namespace}) x 2,)
-    || $Init->plugin->model->{Controllers}->new_controller($Init->{controller}, 'admin actions');
+  my $cc = $Init->plugin->model('Controllers')->controller_ns($Init->{controller}, ($Init->{namespace}) x 2,)
+    || $Init->plugin->model('Controllers')->new_controller($Init->{controller}, 'admin actions');
   
   #Namespace
-  my $ns = $Init->plugin->model->{Namespaces}->namespace(undef, $Init->{namespace},)
-    || $Init->plugin->model->{Namespaces}->new_namespace($Init->{namespace}, 'plugin ns!', undef, undef,);
+  my $ns = $Init->plugin->model('Namespaces')->namespace(undef, $Init->{namespace},)
+    || $Init->plugin->model('Namespaces')->new_namespace($Init->{namespace}, 'plugin ns!', undef, undef,);
   
   #ref namespace -> controller
-  my $nc = $Init->plugin->model->{Refs}->refer($ns->{id}, $cc->{id});
+  my $nc = $Init->plugin->model('Refs')->refer($ns->{id}, $cc->{id});
   
   #REF namespace->role
-  my $cr = $Init->plugin->model->{Refs}->refer($ns->{id}, $rl->{id});
+  my $cr = $Init->plugin->model('Refs')->refer($ns->{id}, $rl->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -156,7 +163,7 @@ TXT
 sub new_role {
 	my $c = shift;
 	my ($name) = $c->vars('name');
-	my $r = $Init->plugin->model->{Roles}->get_role(undef, $name);
+	my $r = $Init->plugin->model('Roles')->get_role(undef, $name);
 	$c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -168,7 +175,7 @@ Role exists
 TXT
 		and return $c
 		if $r;
-	$r = $Init->plugin->model->{Roles}->new_role($name);
+	$r = $Init->plugin->model('Roles')->new_role($name);
 	
 	$c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -185,7 +192,7 @@ TXT
 sub user_roles {
   my $c = shift;
   my ($user) = $c->vars(qw'user login');# || $c->vars('login');
-  my $u =  $Init->plugin->model->{Profiles}->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
+  my $u =  $Init->plugin->model('Profiles')->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -219,7 +226,7 @@ TXT
 
 sub roles {
   my $c = shift;
-  my $r = $Init->plugin->model->{Roles}->roles;
+  my $r = $Init->plugin->model('Roles')->roles;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -236,7 +243,7 @@ sub new_role_user {
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  my $r = $Init->plugin->model('Roles')->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -246,10 +253,10 @@ Can't create new role by only digits[$role] in name
 TXT
     and return
     unless $r && $role =~ /\w/;
-  $r ||= $Init->plugin->model->{Roles}->new_role($role);
+  $r ||= $Init->plugin->model('Roles')->new_role($role);
   
   my ($user) = $c->vars('user');
-  my $u = $Init->plugin->model->{Profiles}->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
+  my $u = $Init->plugin->model('Profiles')->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -261,7 +268,7 @@ TXT
     and return
     unless $u;
   
-  my $ref = $Init->plugin->model->{Refs}->refer($r->{id} => $u->{id});
+  my $ref = $Init->plugin->model('Refs')->refer($r->{id} => $u->{id});
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -280,7 +287,7 @@ sub new_role_controller {
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  my $r = $Init->plugin->model('Roles')->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -290,10 +297,10 @@ Can't create new role by only digits[$role] in name
 TXT
     and return
     unless $r && $role =~ /\w/;
-  $r ||= $Init->plugin->model->{Roles}->new_role($role);
+  $r ||= $Init->plugin->model('Roles')->new_role($role);
   
   my ($ns, $controll) = $c->vars(qw'ns controll');
-  my $cntr = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns);
+  my $cntr = $Init->plugin->model('Controllers')->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns);
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -305,7 +312,7 @@ TXT
     and return
     unless $cntr;
   
-  my $ref = $Init->plugin->model->{Refs}->refer($cntr->{id} => $r->{id},);
+  my $ref = $Init->plugin->model('Refs')->refer($cntr->{id} => $r->{id},);
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -330,7 +337,7 @@ sub del_role_user {# удалить связь пользователя с ро
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  my $r = $Init->plugin->model('Roles')->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -342,7 +349,7 @@ TXT
     unless $r;
 
   my ($user) = $c->vars('user');
-  my $u = $Init->plugin->model->{Profiles}->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
+  my $u = $Init->plugin->model('Profiles')->get_profile($user =~ /\D/ ? (undef, $user) : ($user, undef,));
   
   $c->render(format=>'txt', text=><<TXT)
 $pkg
@@ -354,7 +361,7 @@ TXT
     and return
     unless $u;
   
-  my $ref = $Init->plugin->model->{Refs}->del(undef, $r->{id}, $u->{id});
+  my $ref = $Init->plugin->model('Refs')->del(undef, $r->{id}, $u->{id});
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -382,7 +389,7 @@ sub disable_role {
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $Init->plugin->model->{Roles}->dsbl_enbl($a, $role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  my $r = $Init->plugin->model('Roles')->dsbl_enbl($a, $role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -412,7 +419,7 @@ sub role_users {# все пользователи роли по запросу /
   
   my ($role) = $c->vars('role');
   # ROLE
-  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  my $r = $Init->plugin->model('Roles')->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -423,7 +430,7 @@ TXT
     and return
     unless $r;
   
-  my $u = $Init->plugin->model->{Roles}->profiles($r->{id});
+  my $u = $Init->plugin->model('Roles')->profiles($r->{id});
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -439,7 +446,7 @@ sub role_routes {# все маршруты роли по запросу /myadmin
   
    my ($role) = $c->vars('role');
   # ROLE
-  my $r = $Init->plugin->model->{Roles}->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
+  my $r = $Init->plugin->model('Roles')->get_role($role =~ /\D/ ? (undef, $role) : ($role, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -449,7 +456,7 @@ TXT
     and return
     unless $r;
   
-  my $t = $Init->plugin->model->{Routes}->routes_ref($r->{id});
+  my $t = $Init->plugin->model('Routes')->routes_ref($r->{id});
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -461,7 +468,7 @@ TXT
 
 sub controllers {
   my $c = shift;
-  my $list = $Init->plugin->model->{Controllers}->controllers;
+  my $list = $Init->plugin->model('Controllers')->controllers;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -475,7 +482,7 @@ TXT
 sub controll {# /controller/:ns/:controll
   my $c = shift;
   my ($ns, $controll) = $c->vars(qw(ns controll));
-  my $r = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns);
+  my $r = $Init->plugin->model('Controllers')->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef), $ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef), $ns);
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -492,7 +499,7 @@ sub new_controller {
   my ($ns) = $c->vars(qw'ns namespace');# || ($c->vars('namespace'));
   my ($mod) = $c->vars(qw'module controll');
   my ($descr) = $c->vars(qw'descr');
-  my $cn = $Init->plugin->model->{Controllers}->controller_ns($mod, ($ns) x 2,);
+  my $cn = $Init->plugin->model('Controllers')->controller_ns($mod, ($ns) x 2,);
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -504,11 +511,11 @@ TXT
   and return
   if $cn;
   my $n = $c->new_namespace($ns) if $ns;
-  $cn = $Init->plugin->model->{Controllers}->new_controller($mod, $descr);
-  $Init->plugin->model->{Refs}->refer($n->{id}, $cn->{id})
+  $cn = $Init->plugin->model('Controllers')->new_controller($mod, $descr);
+  $Init->plugin->model('Refs')->refer($n->{id}, $cn->{id})
     if $n;
   
-  $cn = $Init->plugin->model->{Controllers}->controller_ns($mod, ($ns) x 2,);
+  $cn = $Init->plugin->model('Controllers')->controller_ns($mod, ($ns) x 2,);
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -523,7 +530,7 @@ TXT
 
 sub namespaces {
   my $c = shift;
-  my $list = $Init->plugin->model->{Namespaces}->namespaces;
+  my $list = $Init->plugin->model('Namespaces')->namespaces;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -540,7 +547,7 @@ sub new_namespace {
   my ($descr) = $_[0] ? (shift) :  $c->vars('descr');
   my ($app_ns) = $_[0] ? (shift) : $c->vars('app_ns');
   my ($interval_ts) = $_[0] ? (shift) : $c->vars('interval_ts');
-  my $n = $Init->plugin->model->{Namespaces}->namespace($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,));
+  my $n = $Init->plugin->model('Namespaces')->namespace($ns =~ /\D/ ? (undef, $ns) : ($ns, undef,));
   $c->render(format=>'txt', text=><<TXT)
 $pkg
 
@@ -551,7 +558,7 @@ Namespace already exists
 TXT
   and return $n
   if $n;
-  $n = $Init->plugin->model->{Namespaces}->new_namespace($ns, $descr, $app_ns, $interval_ts);
+  $n = $Init->plugin->model('Namespaces')->new_namespace($ns, $descr, $app_ns, $interval_ts);
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -566,9 +573,9 @@ TXT
 
 sub actions {
   my $c = shift;
-  my $list = $Init->plugin->model->{Actions}->actions;
+  my $list = $Init->plugin->model('Actions')->actions;
   map {
-    $_->{routes} = $Init->plugin->model->{Routes}->routes_action($_->{id});
+    $_->{routes} = $Init->plugin->model('Routes')->routes_action($_->{id});
   } @$list;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -582,7 +589,7 @@ TXT
 
 sub routes {
   my $c = shift;
-  my $list = $Init->plugin->model->{Routes}->routes;
+  my $list = $Init->plugin->model('Routes')->routes;
   
   $c->render(format=>'txt', text=><<TXT);
 $pkg
@@ -596,7 +603,7 @@ TXT
 
 sub new_route_ns {# показать список мест-имен
   my $c = shift;
-  my $list = $Init->plugin->model->{Namespaces}->namespaces;
+  my $list = $Init->plugin->model('Namespaces')->namespaces;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
@@ -614,16 +621,18 @@ sub new_route_c {# показать список контроллеров
   no warnings;
   my $c = shift;
   my ($ns) = $c->vars('ns');
-  $ns = $Init->plugin->model->{Namespaces}->namespace($ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
+  $ns = $Init->plugin->model('Namespaces')->namespace($ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
     || {namespace => $ns};
-  my $list = $Init->plugin->model->{Controllers}->controllers_ns_id($ns->{id}, $ns->{id});
+  my $list = $ns->{id} ? $Init->plugin->model('Controllers')->controllers_ns_id($ns->{id}, $ns->{id}) : ['спейс пропускается'];
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-1. namespace = [$ns->{id}:$ns->{namespace}]
+1. namespace = [@{[$c->dumper( $ns )]}]
 
 Указать имя или ID контроллера или ввести новое имя
 
+Если указать undef для последующей неявной привязкой через параметр маршрута - "to"
+
 Controllers (@{[scalar @$list]})
 ===
 
@@ -636,22 +645,24 @@ sub new_route_a {# показать список действий
   my $c = shift;
   my ($ns, $controll) = $c->vars(qw'ns controll');
   
-  $ns = $Init->plugin->model->{Namespaces}->namespace($ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
+  $ns = ($ns && $Init->plugin->model('Namespaces')->namespace($ns && $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
     || {namespace => $ns};
   
-  $controll = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), $ns->{id}, $ns->{namespace}, $ns->{namespace},)
+  $controll = ($controll && $Init->plugin->model('Controllers')->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), $ns->{id}, $ns->{namespace}, $ns->{namespace},))
     || {controller=>$controll};
   
-  my $list = $Init->plugin->model->{Actions}->actions_controller($controll->{id});
-  my $list2 = $Init->plugin->model->{Actions}->actions_controller_null();
+  my $list = $controll->{id} ? $Init->plugin->model('Actions')->actions_controller($controll->{id}) : ['контроллер пропускается'];
+  my $list2 = $Init->plugin->model('Actions')->actions_controller_null();
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-1. namespace = [$ns->{id}:$ns->{namespace}]
-2. controller = [$controll->{id}:$controll->{controller}]
+1. namespace = [@{[$c->dumper( $ns )]}]
+2. controller = [@{[$c->dumper( $controll )]}]
 
 Указать имя или ID действия из списка или ввести новое имя действия
 
+Или указать undef чтобы привязать маршрут к контроллеру или вообще неявно через параметр route"to", но тогда на следующем шаге обязательно указать параметр to=->имя метода в котроллере 
+
 Actions for selected controller (@{[scalar @$list]})
 ===
 @{[$c->dumper( $list )]}
@@ -663,19 +674,19 @@ Actions without controller (@{[scalar @$list2]}):
 TXT
 }
 
-my @route_cols = qw(request name descr auth disable interval_ts);
+my @route_cols = qw(request to name descr auth disable interval_ts);
 sub new_route {# показать маршруты к действию
   my $c = shift;
   my ($ns, $controll, $act) = $c->vars(qw'ns controll act');
   
-  $ns = $Init->plugin->model->{Namespaces}->namespace($ns&& $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,))
+  $ns = ($ns && $Init->plugin->model('Namespaces')->namespace($ns&& $ns =~ /\D/ ? (undef, $ns) : ($ns, undef,)))
     || {namespace => $ns};
   
-  $controll = $Init->plugin->model->{Controllers}->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), $ns->{id}, $ns->{namespace}, $ns->{namespace},)
+  $controll = ($controll && $Init->plugin->model('Controllers')->controller_id_ns($controll =~ /\D/ ? (undef, $controll) : ($controll, undef,), $ns->{id}, $ns->{namespace}, $ns->{namespace},))
     || {controller=>$controll};
   
-  $act = $Init->plugin->model->{Actions}->action_controller($controll->{id}, $act =~ /\D/ ? (undef, $act) : ($act, undef,),)
-    || $Init->plugin->model->{Actions}->action_controller_null($act =~ /\D/ ? (undef, $act) : ($act, undef,),)
+  $act = ($act && $Init->plugin->model('Actions')->action_controller($controll->{id}, $act =~ /\D/ ? (undef, $act) : ($act, undef,),))
+    || ($act && $Init->plugin->model('Actions')->action_controller_null($act =~ /\D/ ? (undef, $act) : ($act, undef,),))
     || {action => $act};
   
   # Проверка на похожий $request ?? TODO
@@ -716,36 +727,37 @@ TXT
   
   
   # маршруты действия
-  my $list = $act->{id} ? $Init->plugin->model->{Routes}->routes_action($act->{id})
-    : [];
+  my $list = $act->{id} ? $Init->plugin->model('Routes')->routes_action($act->{id})
+    : ['нет явного действия'];
   # свободные маршруты
-  my $list2 = $Init->plugin->model->{Routes}->routes_action_null;
+  my $list2 = $Init->plugin->model('Routes')->routes_action_null;
   
   no warnings;
   $c->render(format=>'txt', text=><<TXT);
 $pkg
 
-1. namespace = [$ns->{id}:$ns->{namespace}]
-2. controller = [$controll->{id}:$controll->{controller}]
-3. action = [$act->{id}:$act->{action}]
+1. namespace = [@{[$c->dumper( $ns )]}]
+2. controller = [@{[$c->dumper( $controll )]}]
+3. action = [@{[$c->dumper( $act )]}]
 
-Указано: 
-@{[map ("$_=$route->{$_}\n", @route_cols)]}
+Маршрут: 
+@{[$c->dumper( $route )]}
 
 Указать параметры маршрута (?request=/x/y/:z&name=xyz&descr=...):
 
 * request (request=GET POST /foo/:bar)
 * name (name=foo_bar)
+- to (to=Foo->bar or to=->bar)
 - descr (descr=пояснение такое)
 - auth (auth=1) (auth='only')
 - disable (disable=1)
 - interval_ts (interval_ts=123)
 
-Exists routes for selected action (@{[$list ? scalar @$list : 0]})
+Exists routes for selected action (@{[scalar @$list ]})
 ===
 @{[$c->dumper( $list )]}
 
-Free routes (@{[$list2 ? scalar @$list2 : 0]})
+Free routes (@{[scalar @$list2]})
 ===
 @{[$c->dumper( $list2 )]}
 
@@ -756,21 +768,24 @@ TXT
 sub route_save {
   my $c = shift;
   my ($ns, $controll, $act, $route) = @_;
-  local $Init->plugin->dbh->{AutoCommit} = 0;
-  $ns = $Init->plugin->model->{Namespaces}->new_namespace(@$ns{qw(namespace descr app_ns interval_ts)})
+  #~ local $Init->plugin->dbh->{AutoCommit} = 0;
+  $ns = $Init->plugin->model('Namespaces')->new_namespace(@$ns{qw(namespace descr app_ns interval_ts)})
     if $ns->{namespace} && ! $ns->{id};
-  $controll = $Init->plugin->model->{Controllers}->new_controller(@$controll{qw(controller descr)})
-    unless $controll->{id};
-  $act = $Init->plugin->model->{Actions}->new_action(@$act{qw(action callback descr)})
-    unless $act->{id};
-    
-  $route = $Init->plugin->model->{Routes}->new_route(@$route{@route_cols})
+  $controll = $Init->plugin->model('Controllers')->new_controller(@$controll{qw(controller descr)})
+    if $controll->{controller} && !$controll->{id};
+  $act = $Init->plugin->model('Actions')->new_action(@$act{qw(action callback descr)})
+    if $act->{action} && !$act->{id};
+  
+  $route->{to} =~ s/->/#/
+    if $route->{to};
+  
+  $route = $Init->plugin->model('Routes')->new_route(@$route{@route_cols})
     unless $route->{id};
   my $ref = [map {
-    $Init->plugin->model->{Refs}->refer($$_[0]{id}, $$_[1]{id},)
+    $Init->plugin->model('Refs')->refer($$_[0]{id}, $$_[1]{id},)
       if $$_[0]{id} && $$_[1]{id};
-  } ([$ns, $controll], [$controll, $act], [$route, $act],)];
-  $Init->plugin->dbh->commit;
+  } ([$ns, $controll], [$controll, $act], [ $act, $route,], $act->{id} ? () : [$controll, $route,])];
+  #~ $Init->plugin->dbh->commit;
   return ($ns, $controll, $act, $route, $ref);
 
 }
@@ -785,21 +800,16 @@ sub vars {# получить из stash || param
   } @_;
 }
 
-#~ sub ref {# get or save
-  #~ my $c = shift;
-  #~ my ($id1, $id2) = @_;
-  #~ $Init->plugin->model->{Refs}->refer($id1, $id2,);
-#~ }
-
-
-
 my @self_routes_cols = qw(request action name auth descr);
 sub self_routes {# from plugin!
   my $c = shift;
   my $prefix = $Init->prefix;
   my $trust = $Init->trust;
+  my $role_admin = $Init->role_admin;
 
   my $t = <<TABLE;
+#format
+#<route path>\t<method>\t<route name>\t<need auth>\tDescription
 /$prefix	index	admin home	1	Main page
 #
 # Namespaces, controllers, actions
@@ -847,9 +857,12 @@ sub self_routes {# from plugin!
 #get foo /sign/in	sign	signin form	0	Login&pass form
 #post /sign/in	sign	signin params	0	Auth by params
 /$prefix/sign/in/:login/:pass	sign	signin stash	0	Auth by stash
-/$prefix/sign/out	signout	go away	1	Exit
+/$prefix/sign/out	signout	go away	only	Exit
+/logout	out	logout	only	Exit and redirect
+/reauth/:cookie	auth_cookie	auth-cookie	0	Relogin by cookie
 #
-/$prefix/$trust/admin/new/:login/:pass	trust_new_user	$prefix/$trust !trust create user!	0	Add new user by :login & :pass and auto assign to role 'Admin' and assign to access this controller!
+
+/$prefix/$trust/$role_admin/new/:login/:pass	trust_new_user	$prefix/$trust !trust create user!	0	Add new user by :login & :pass and auto assign to role 'Admin' and assign to access this controller!
 
 TABLE
   
@@ -866,6 +879,30 @@ TABLE
   return @r;
 }
 
+sub auth_cookie {# action
+  my $c = shift;
+  my $json = $c->req->json;
+  
+  my ($cookie) = ($json && $json->{cookie})
+    || $c->vars('cookie');
+  
+  unless ($cookie) {
+    return $c->render(json=>{error=>"No cookie"})
+      if $json;
+    
+    return $c->render(text=>"Error: no cookie");
+    
+  }
+  
+  my $profile = { %{$c->access->auth_cookie($c, $cookie) || {error=>"none profile by given cookie [$cookie]"}} };
+  
+  return $c->render(json=>{profile=>$profile})
+    if $json;
+  
+  return $c->render(text=>"Success signed by cookie\n".$c->dumper($profile));
+  
+}
+
 1;
 
 =pod
@@ -878,7 +915,7 @@ TABLE
 
 =head1 WARN
 
-More or less complete!
+More or less complete! :)
 
 =head1 NAME
 
@@ -917,6 +954,9 @@ String. Is a prefix for admin urls of this module. Default as name of controller
 
 String. Is a url subprefix for trust admin urls of this module. See defaults below.
 
+=head3 role_admin
+
+String. Is a name of role for admonistrators.
 
 =head2 Default options
 
@@ -925,6 +965,7 @@ String. Is a url subprefix for trust admin urls of this module. See defaults bel
     controller => 'Admin',
     prefix => 'admin', # lc(<module>)
     trust => hmac_sha1_sum('admin', $app->secrets->[0]),
+    role_admin => 'administrators',
   },
 
 Examples options:
@@ -939,18 +980,19 @@ Examples options:
 
 =head1 METHODS NEEDS IN PLUGIN
 
-=over 4
+=head2 self_routes()
 
-=item * B<self_routes()> - builtin to this access controller routes. Return array of hashrefs routes records for apply route on app. Depends on conf options I<prefix> and I<trust>.
+Builtin to this admin controller routes. Return array of hashrefs routes records for apply route on app. Depends on conf options I<prefix> and I<trust>.
 
-=back
+
+=head1 ROUTES
+
+There are number of app routes on this controller. See in console C< $perl your-app.pl routes >. That routes will not apply then admin controller disabled.
 
 =head1 SEE ALSO
 
 L<Mojolicious::Plugin::RoutesAuthDBI>
 
-L<Mojolicious::Plugin::RoutesAuthDBI::Sth>
-
 =head1 AUTHOR
 
 Михаил Че (Mikhail Che), C<< <mche [on] cpan.org> >>
@@ -966,5 +1008,4 @@ Copyright 2016 Mikhail Che.
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-=cut
-
+=cut
\ No newline at end of file
