@@ -16,6 +16,7 @@ our $defaults = {# copy to pod!
     namespaces => 'namespaces',
     oauth_sites => 'oauth.sites',
     oauth_users => 'oauth.users',
+    guests => 'guests',
   },
   
 };
@@ -55,6 +56,7 @@ See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagr
     namespaces => 'namespaces',
     oauth_sites => 'oauth.sites',
     oauth_users => 'oauth.users',
+    guests => 'guests',
     },
   }
 
@@ -64,77 +66,55 @@ See L<https://github.com/mche/Mojolicious-Plugin-RoutesAuthDBI/blob/master/Diagr
 
 sub _vars {
   my $c = shift;
-  my $template = {};
+  my $vars = {};
 
   for my $var (keys %$defaults) {
-    my $val = $c->stash($var) || $c->param($var);
-    $template->{$var} = $val
-      and next
-      if $val;
-      
-    $template->{$var} = { map {
-      my $val = $c->stash($_) || $c->param($_);
-      $val ? ($_ => $val) : ();
     
-    } keys %{$defaults->{$var}}  }
-      if ref $defaults->{$var};
+    if (ref $defaults->{$var} eq 'HASH') {
+      
+      for (keys %{$defaults->{$var}}) {
+        my $val = $c->stash($_) || $c->param($_) || $defaults->{$var}{$_};
+        $vars->{$var}{$_} = $val
+          unless $val =~ m'undef|none';
+      }
+    } else {
+      my $val = $c->stash($var) || $c->param($var) || $defaults->{$var};
+      $vars->{$var} = $val
+        unless $val =~ m'undef|none';
+    }
+    
   }
-  $template;
+  $vars;
 }
 
 sub schema {
   my $c = shift;
-  my $template = $c->_vars;
+  my $vars = $c->_vars;
   
-  $c->app->log->debug($c->dumper($template));
+  $c->app->log->debug($c->dumper($vars));
   
-  #~ my $schema2 = qq{"$schema".} if $schema;
-  $c->render(format=>'txt', text => <<TXT);
-@{[$dict->{'schema'}->template(%$template)]}
+  my $text = $dict->{'schema'}->template(%$vars);
+  $text .= "\n\n".$dict->{'sequence'}->template(%$vars)
+    if $vars->{sequence};
+  $text .= "\n\n".$dict->{$_}->template(%$vars)
+    for grep $vars->{tables}{$_}, qw(routes namespaces controllers actions profiles logins roles refs oauth_sites oauth_users guests);
+  
+  $c->render(format=>'txt', text => $text);
 
-@{[$dict->{'sequence'}->template(%$template)]}
-
-@{[$dict->{'routes'}->template(%$template)]}
-
-@{[$dict->{'namespaces'}->template(%$template)]}
-
-@{[$dict->{'controllers'}->template(%$template)]}
-
-@{[$dict->{'actions'}->template(%$template)]}
-
-@{[$dict->{profiles}->template(%$template)]}
-
-@{[$dict->{'logins'}->template(%$template)]}
-
-@{[$dict->{'roles'}->template(%$template)]}
-
-@{[$dict->{'refs'}->template(%$template)]}
-
-@{[$dict->{'oauth_sites'}->template(%$template)]}
-
-@{[$dict->{'oauth_users'}->template(%$template)]}
-
-TXT
 }
 
 sub schema_drop {
   my $c = shift;
-  my $template = $c->_vars;
+  my $vars = $c->_vars;
   #~ $schema = qq{"$schema".} if $schema;
-  $c->render(format=>'txt', text => <<TXT);
-@{[$dict->{'drop'}->template(%$template)]}
+  $c->render(format=>'txt', text => $dict->{'drop'}->template(%$vars));
 
-TXT
 }
 
 sub schema_flush {
   my $c = shift;
-  my $template = $c->_vars;
-  $c->render(format=>'txt', text => <<TXT);
-
-@{[$dict->{'flush'}->template(%$template)]}
-
-TXT
+  my $vars = $c->_vars;
+  $c->render(format=>'txt', text => $dict->{'flush'}->template(%$vars));
 }
 
 1;
@@ -156,7 +136,8 @@ CREATE SEQUENCE {%= $sequence %};-- one sequence for all tables id
 CREATE TABLE "{%= $schema %}"."{%= $tables->{routes} %}" (
   id integer default nextval('{%= $sequence %}'::regclass) not null primary key,
   ts timestamp without time zone default now() not null,
-  request character varying not null,
+  request character varying not null unique, -- 'get /foo/bar/:x'
+  "to" character varying, -- 'Foo#bar'
   name character varying not null unique,
   descr text null,
   auth varchar null,
@@ -259,6 +240,15 @@ create table IF NOT EXISTS "{%= $schema %}"."{%= $tables->{oauth_users} %}" (
   unique (site_id, user_id)
 );
 
+@@ guests
+-- Гостевые записи
+create table IF NOT EXISTS "{%= $schema %}"."{%= $tables->{guests} %}" (
+  id integer NOT NULL DEFAULT nextval('{%= $sequence %}'::regclass) primary key,
+  ts timestamp without time zone NOT NULL DEFAULT now(),
+  data jsonb null
+  
+);
+
 @@ drop
 --drop table "{%= $schema %}"."{%= $tables->{refs} %}" cascade;
 --drop table "{%= $schema %}"."{%= $tables->{logins} %}" cascade;
